<?php

namespace Aalogics\Parcelninja\Gateway;

use Aalogics\Parcelninja\Gateway\AbstractGateway;
use Aalogics\Parcelninja\Model\GatewayInterface;
use Magento\Quote\Model\Quote\Address\RateRequest;
use Aalogics\Parcelninja\Gateway\Parcelninja\Request as ParcelninjaRequest;
use Magento\Catalog\Model\Product\Type;
use Aalogics\Parcelninja\Helper\Data;


class Parcelninja extends AbstractGateway implements GatewayInterface
{

    const DELIVERY_METHOD = "carriers/aalogics_parcelninja/delivery_method";
    
    protected $_request;

    protected $ParcelninjaRequest;

    protected $helperData;

    /**
     * @var \Magento\Shipping\Model\Rate\ResultFactory
     */
    protected $_rateResultFactory;

    /**
     * @var \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory
     */
    protected $_rateMethodFactory;

    protected $addressInterface;
  
    protected $scopeConfigObject;

    public function __construct(
        ParcelninjaRequest $ParcelninjaRequest,
        Data $helperData,
        \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        \Magento\Quote\Api\Data\AddressInterface $addressInterface,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ){
        $this->_rateResultFactory = $rateResultFactory;
        $this->_rateMethodFactory = $rateMethodFactory;
        $this->helperData = $helperData;
        $this->ParcelninjaRequest = $ParcelninjaRequest;
        $this->addressInterface = $addressInterface;
        $this->scopeConfigObject = $scopeConfig;
    }





    public function getDeliveryMethod($store = null){

      return $this->scopeConfigObject->getValue(
              self::DELIVERY_METHOD,
              \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
              $store
          );

    }

	/**
     * Returns array of Quotes for Shipping
     * @return array
     */
	public function getQuotes(RateRequest $request){

		$this->_request = $request;

    $shippingInformation = $this->_getShippingInformation();
    $deliveryMethod = $this->getDeliveryMethod();

    if(is_null($shippingInformation['postalCode'])){
      $quotes = [];
    }
    else{
      if($deliveryMethod=="normal"){
        $quotes = $this->ParcelninjaRequest->sendRequest(
            \Aalogics\Parcelninja\Gateway\Parcelninja\Endpoints\GetQuotes::class
            ,
            [
                'deliveryInformation' => $this->_getShippingInformation(),
                'items'               => $this->_getAllItems()
            ]
        );
      }
      else if($deliveryMethod=="cheapest"){
        $quotes = $this->ParcelninjaRequest->sendRequest(
            \Aalogics\Parcelninja\Gateway\Parcelninja\Endpoints\GetCheapestQuotes::class
            ,
            [
                'deliveryInformation' => $this->_getShippingInformation(),
                'items'               => $this->_getAllItems()
            ]
        );
      }
      else if($deliveryMethod=="fastest"){
        $quotes = $this->ParcelninjaRequest->sendRequest(
            \Aalogics\Parcelninja\Gateway\Parcelninja\Endpoints\GetFastestQuotes::class
            ,
            [
                'deliveryInformation' => $this->_getShippingInformation(),
                'items'               => $this->_getAllItems()
            ]
        );
      }
    }

    return $quotes;

	}

  /**
     * Returns array of Response for Testing
     * @return array
     */
  public function getResponse(){
    
        $response = $this->ParcelninjaRequest->sendRequest(
            \Aalogics\Parcelninja\Gateway\Parcelninja\Endpoints\GetResponse::class
            ,
            [
                    'deliveryInformation' =>   array("postalCode"=> "2196","suburb"=> "Bryanston"),
                    'items'               =>   array(array("sku"=>"testSku","quantity"=>1,"dimensions"=>array("width"=>100,"length"=>100,"height"=>100,"weight"=>100)))
            ]
        );   
    return $response;

  }

	/**
     * Returns array of shipping Information
     * @return array
     */
	protected function _getShippingInformation()
    {
        $shippingInformation = [
            'postalCode'    =>  $this->_request->getDestPostcode(),
            'suburb'        =>  $this->_request->getDestCity()?$this->_request->getDestCity():$this->_request->getDestRegionCode()
        ];

        return $shippingInformation;
    }

    /**
     * Prepare items to pieces
     *
     * @return array
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * phpcs:disable Generic.Metrics.NestingLevel
     */
    protected function _getAllItems()
    {
        $allItems = $this->_request->getAllItems();
        $fullItems = [];

        foreach ($allItems as $item) {

            if ($item->getProductType() == Type::TYPE_BUNDLE && $item->getProduct()->getShipmentType()) {
                continue;
            }

            $qty = $item->getQty();
            $changeQty = true;
            $checkWeight = true;
            $decimalItems = [];

            if ($item->getParentItem()) {
                if (!$item->getParentItem()->getProduct()->getShipmentType()) {
                    continue;
                }
                if ($item->getIsQtyDecimal()) {
                    $qty = $item->getParentItem()->getQty();
                } else {
                    $qty = $item->getParentItem()->getQty() * $item->getQty();
                }
            }

            $dimensions = $this->helperData->getDimension();

            

            $dimensions['weight'] = ($item->getData('row_weight') > 1) ? $item->getData('row_weight') : $dimensions['weight'];

            $temparray = [
                "sku" => $item->getSku(),
                  "quantity"    => $qty,
                  "dimensions"  => $dimensions,
                  "fromReserve"=> FALSE
            ];

            $fullItems[] = $temparray;
            
        }

        return $fullItems;
    }

}